# Author: Andrew.M.G.Reynen
from __future__ import print_function,division
import os
import sys
import ctypes as C
if sys.version_info[0]==2:
    from scandir import scandir
else:
    from os import scandir
    
import numpy as np
from obspy.core.stream import read
from obspy.core.stream import Stream as EmptyStream
from obspy.core.utcdatetime import UTCDateTime
from obspy.io.mseed.headers import clibmseed,VALID_CONTROL_HEADERS,SEED_CONTROL_HEADERS
from PyQt5 import QtWidgets, QtCore

# If the stream was not able to be merged, check to see if multiple sampling rates on the same channel...
# ... and remove the ones with the uncommon sampling rate
def RemoveOddRateTraces(stream):
    unqStaRates=[] # Array to hold rates, per channel
    rates,chas=[],[]
    for tr in stream:
        anID=tr.stats.station+'.'+tr.stats.channel+'.'+str(tr.stats.delta)
        if anID not in unqStaRates:
            unqStaRates.append(anID)
            rates.append(tr.stats.delta)
            chas.append(tr.stats.station+'.'+tr.stats.channel)
    # ...figure out which station has two rates 
    unqRate,countRate=np.unique(rates,return_counts=True)
    if len(unqRate)==1:
        print('merge will fail, not issue of multiple sampling rates on same station')
    else:
        # ... and remove the traces with less common rates
        unqCha,countCha=np.unique(chas,return_counts=True)
        rmChas=unqCha[np.where(countCha!=1)]
        rmRates=unqRate[np.where(unqRate!=(unqRate[np.argmax(countRate)]))]
        trimRateStream=EmptyStream()
        for tr in stream:
            if tr.stats.station+'.'+tr.stats.channel in rmChas and tr.stats.delta in rmRates:
                continue
            trimRateStream+=tr
        print('station.channel:',str(rmChas),'had some traces removed (duplicate rates same channel)')
        stream=trimRateStream
    return stream

# Given two timestamps, extract all information...
# fileTimes contains [startTime,endTime] of the archive files
def extractDataFromArchive(t1,t2,fileNames,fileTimes,wantedStaChas=[['*','*']]):
    # Return nothing if there is no data
    if len(fileTimes)==0:
        return EmptyStream()
    # Catch the case where the asked time range is completely outside the archive data availability
    if t1>fileTimes[-1,1] or t2<fileTimes[0,0]:
        return EmptyStream()
    # Figure out what set of files are wanted
    collectArgs=np.where((fileTimes[:,0]<=t2)&(fileTimes[:,1]>=t1))[0]
    stream=EmptyStream()
    flagged=False
    # Read in all of the information
    for aFile in fileNames[collectArgs]:
        # Flag to user if the archive structure has changed
        if not os.path.exists(aFile):
            flagged=True
            continue
        aStream=read(aFile)
        for aSta,aCha in wantedStaChas:
            stream+=aStream.select(station=aSta,channel=aCha)
    if flagged: 
        print('Archive structure as changed, reload the current archive')
    # Merge traces which are adjacent
    try:
        stream.merge(method=1)
    except:
        stream=RemoveOddRateTraces(stream)
        stream.merge(method=1)
    # If any trace has masked values, split
    if True in [isinstance(tr.data, np.ma.masked_array) for tr in stream]:
        stream=stream.split()
    # Trim to wanted times
    stream.trim(UTCDateTime(t1),UTCDateTime(t2))
    return stream
    
# Get a list of all files of the accepted extension types, and their metadata
def getDirFiles(mainDir,acceptedExtensions):
    metaData=[] # The returned metadata [path,mtime,ctime,fileSize]
    # Check first to see that the folder exists, return nothing otherwise
    if not os.path.isdir(mainDir):
        return metaData
    # The directories yet to be scanned
    toScanDirs=[mainDir]
#    print('Scanning archive directory for changes...')
    while len(toScanDirs)!=0:
        toAddToScan=[] # Directories to be found in this iteration
        for aDir in toScanDirs:
            for entry in scandir(aDir):
                # If this is a directory, scan it later
                if entry.is_dir():
                    toAddToScan.append(entry.path)
                    continue
                # Add to the file metadata list if is an accepted file type
                if entry.name.split('.')[-1] in acceptedExtensions:
                    aStat=entry.stat()
                    metaData.append([entry.path,aStat.st_mtime,
                                     aStat.st_ctime,aStat.st_size])
        # Add all of the new directories which were found
        toScanDirs=toAddToScan
#    print('...scan complete')
    return metaData

# Get the previously loaded archives file metadata and start/end times
def getPrevArchive(mainDir):
    try:
        meta=np.load(mainDir+'/lazylystArchiveMeta.npy')
        times=np.load(mainDir+'/lazylystArchiveTimes.npy')
        return meta,times
    except:
        return np.empty((0,4)),np.empty((0,2))
    
# Empty class to be able to pass objects to the progress widget
class ValueObj(object):
    def __init__(self):
        self.value=[]

# Read in all the start and stop times of the files
def getArchiveAvail(archDir,acceptFileTypes=['seed','miniseed','mseed'],showBar=True):
    # Get the currently present files metadata
    curMeta=getDirFiles(archDir,acceptFileTypes)
    curMeta=np.array(curMeta,dtype=str)
    curTimes=np.zeros((len(curMeta),2))
    # If there are no files in archDir
    if len(curMeta)==0:
        return np.array([]),curTimes
    # Get the previously loaded archive metadata if it exists
    prevMeta,prevTimes=getPrevArchive(archDir)
    # If the arrays are already exactly the same, do not bother updating
    if np.array_equal(prevMeta,curMeta):
        return prevMeta[:,0],prevTimes
    # See which files are common among current and previous metadata (check, which may be added to load)...
    prevFiles=list(prevMeta[:,0])
    if len(prevMeta)!=0:
        checkIdxs=[[i,prevFiles.index(item)] for i, item in enumerate(curMeta[:,0]) if item in set(prevFiles)]
    else:
        checkIdxs=[]
    checkIdxs=np.array(checkIdxs)
    # ...as well as which files were not previously present (load)
    if len(checkIdxs)==0:
        loadIdxs=list(range(len(curMeta)))
    else:
        loadIdxs=[i for i in range(len(curMeta)) if i not in checkIdxs[:,0]]
    # Set the current time to be the previous times (where present) by default
    if len(checkIdxs)!=0:
        curTimes[checkIdxs[:,0]]=prevTimes[checkIdxs[:,1]]
    # See which current files need to be loaded
    loadIdxs+=[i for i,j in checkIdxs if not np.array_equal(curMeta[i],prevMeta[j])]
    loadIdxs=np.array(loadIdxs,dtype=int)
    # Go get all of these times ## ##
    holder=ValueObj() 
    if len(loadIdxs)!=0:
        # Have the option to show the progress bar, or run all and now show progress
        bar=ArchLoadProgressBar(holder,curMeta[loadIdxs,0],useTimer=showBar)
        if showBar:
            bar.exec_()
        else:
            while len(holder.value)==0:
                bar.getFileLims()
        times=np.array(holder.value)
        # As the new times loading could have been canceled, update just the ones which were updated
        curTimes[loadIdxs[:len(times)]]=times                   
    
    # Keep the times which are not still [0,0]
    keepIdxs=np.where((curTimes[:,0]!=0)|(curTimes[:,1]!=0))[0]
    curMeta=curMeta[keepIdxs]
    curTimes=curTimes[keepIdxs]
    
    # Finally save the new metadata and times...
    np.save(archDir+'/lazylystArchiveMeta',curMeta)
    np.save(archDir+'/lazylystArchiveTimes',curTimes)
    # ...and return the file names and times
    return curMeta[:,0],curTimes

# Progress bar for the archive...
class ArchLoadProgressBar(QtWidgets.QDialog):
    def __init__(self,holder,toReadFiles,useTimer=True, parent=None):
        super(ArchLoadProgressBar, self).__init__(parent)
        self.holder=holder
        # Set up values to for scanning file times
        self.files=toReadFiles
        self.nFiles=float(len(self.files))
        self.fileMinMaxs=[]
        self.nextFileIdx=0
        # Marker if the quick MSEED reading fails
        self.quickReadFail=False
        # Set up progress bar size and min/max values
        self.resize(300,40)
        self.progressbar = QtWidgets.QProgressBar()
        self.progressbar.setMinimum(1)
        self.progressbar.setMaximum(100)
        # Add cancel button
        self.button = QtWidgets.QPushButton('Cancel')
        self.button.clicked.connect(self.cancelLoad)
        main_layout = QtWidgets.QVBoxLayout()
        # Add widgets and text to the  layout
        main_layout.addWidget(self.progressbar)
        main_layout.addWidget(self.button)
        self.setLayout(main_layout)
        self.setWindowTitle('Loading archive changes...')
        # Add a timer and start loading
        self.timer = QtCore.QTimer()
        if useTimer:
            self.timer.timeout.connect(self.getFileLims)
            self.timer.start(5) # 5 millisecond interval
    
    # Stop the timer from loading files
    def cancelLoad(self):
        self.timer.stop()
        self.holder.value=self.fileMinMaxs
        print(len(self.fileMinMaxs),'archive files were updated')
        self.close()

    # If closed, treat as canceled
    def closeEvent(self, event):
        if self.timer.isActive():
            self.cancelLoad()

    # Get the earliest and latest time data exists in the wanted files
    def getFileLims(self):
        # Stop if no more files to load
        if self.nextFileIdx>=self.nFiles:
            self.cancelLoad()
            return
        nextFile=self.files[self.nextFileIdx]
        # Try the quick read function
        if not self.quickReadFail:
            try:
                startEnds=getMseedStartEnds(nextFile)
                minTime,maxTime=np.min(startEnds[:,0]),np.max(startEnds[:,1])
            except:
                print('Quick MSEED reading failed on file: '+nextFile)
                self.quickReadFail=True
        if self.quickReadFail:
            try:
                st=read(nextFile,headonly=True,format='MSEED')
                stats=[tr.stats for tr in st]
                # Get min and max times from file
                minTime=np.min([stat.starttime for stat in stats]).timestamp
                maxTime=np.max([stat.endtime for stat in stats]).timestamp
            except:
                print('Regular MSEED reading failed on file: '+nextFile)
                minTime,maxTime=0,0 # Assign 0,0 to be removed later
        # Get a measure of progress and update the GUI
        perc=int(100*(self.nextFileIdx+1)/self.nFiles)
        if perc>self.progressbar.value():
            self.progressbar.setValue(perc)
        # Update the GUI every Xth loop
        if self.nextFileIdx%10==0:
            QtWidgets.qApp.processEvents()
        # Update the index, and add this min/max time
        self.nextFileIdx+=1
        self.fileMinMaxs.append([minTime,maxTime])

# Functions used for getMseedStartEnds
def passFunc(*args):
    pass
def isdigit(x):
    return True if (x - ord('0')).max() <= 9 else False

# Read the just start and end time from all traces in MSEED
def getMseedStartEnds(aFile):
    # Read file
    bfr_np = np.fromfile(aFile, dtype=np.int8)
 
    # Parameters wanted for reading just the header
    unpack_data,verbose,offset=0,0,0
    details,selections=False,None
    reclen,header_byteorder=-1,-1

    # Check if starting position of data read should change
    while True:
        if (isdigit(bfr_np[offset:offset + 6]) is False) or \
                (bfr_np[offset + 6] not in VALID_CONTROL_HEADERS):
            msg = 'Not a valid (Mini-)SEED file'
            raise Exception(msg)
        elif bfr_np[offset + 6] in SEED_CONTROL_HEADERS:
            # Get the record length
            try:
                record_length = pow(2, int(''.join([chr(_i) for _i in bfr_np[19:21]])))
            except ValueError:
                record_length = 4096
            offset += record_length
            continue
        break
    bfr_np = bfr_np[offset:]
    buflen = len(bfr_np)

    # Make useless functions to accomodate clibmseed.readMSEEDBuffer requirements
    alloc_data = C.CFUNCTYPE(C.c_longlong, C.c_int, C.c_char)(passFunc)
    log_print = C.CFUNCTYPE(C.c_void_p, C.c_char_p)(passFunc)

    # Read the data with C function
    inputs=[bfr_np, buflen, selections, C.c_int8(unpack_data),
            reclen, C.c_int8(verbose), C.c_int8(details), header_byteorder,
            alloc_data,log_print,log_print]
    # Allow for different versions of obspy
    try:
        lil = clibmseed.readMSEEDBuffer(*inputs)
    except:
        lil = clibmseed.readMSEEDBuffer(*inputs[:-2])
        
    
    # Read first section of the miniseed
    current_id = lil.contents
    timeInfo=np.zeros((0,2),dtype=float)
    # Loop over each NSLC
    while True:
        try:
            current_segment = current_id.firstSegment.contents
        except ValueError:
            break
        # Loop over traces for this NSLC
        while True:
            # Get the starttime and duration
            startStr=str(current_segment.starttime)
            startTime,duration=startStr[:-6]+'.'+startStr[-6:],(1./current_segment.samprate)*(current_segment.samplecnt)
            try:
                current_segment = current_segment.next.contents
            except ValueError:
                # Append these values to array
                timeInfo=np.vstack((timeInfo,np.array([startTime,duration],dtype=float)))
                break
        try:
            current_id = current_id.next.contents
        except ValueError:
            break

    # Remove the mseed info from memory
    clibmseed.lil_free(lil)
    del lil
    
    # Return start and end times
    if len(timeInfo)==0:
        return timeInfo
    else:
        # Make the second entry the end time (start time + duration)
        timeInfo[:,1]=np.sum(timeInfo,axis=1)
        return timeInfo


# Author: Andrew.M.G.Reynen

from obspy import read_inventory
import numpy as np
import pyproj

# Convert a station xml file to a numpy array, containing only [StaCode,Lon,Lat,Ele]
def staXml2Loc(staXml):
    # Loop through just to extract the station codes and locations...
    # ...with format [Net.Sta.Loc,Lon(deg),Lat(deg),Ele(m)], lon/lat assumed to be in WGS84 datum
    outArr=np.empty((0,4),dtype='a32')
    for net in staXml:
        for sta in net:
            if len(sta.channels)==0:
                print(net.code+','+sta.code+' has no channels, could not read location code')
            for cha in sta:
                nsl='.'.join([net.code,sta.code,cha.location_code])
                # Do not add this station id already present
                if len(outArr)>0:
                    if nsl in outArr[:,0]:
                        continue
                entry=np.array([nsl,cha.longitude,cha.latitude,cha.elevation],dtype='a32')
                outArr=np.vstack((outArr,entry))
    # If the file was empty, convert to appropriate shape
    if len(outArr)==0:
        outArr=np.empty((0,4),dtype='a32')
    else:
        outArr=np.array(outArr,dtype=str)
    return outArr

# Read in a station xml file given the path name
def readInventory(staFile):
    return read_inventory(staFile,format='stationxml')

# Project the stations from Lon/Lat to desired system   
def projStaLoc(staLoc,projStyle):
    # If no entries, nothing to change
    if 0 in staLoc.shape:
        return staLoc
    # Generate the projection object to do the conversions...
    # ...Universal Transverse Mercator
    if projStyle=='UTM':
        UTM_Zone=int(np.floor((np.median(staLoc[:,1].astype(float)) + 180.0)/6) % 60) + 1
        wantProj = pyproj.Proj(proj='utm',zone=UTM_Zone,datum='WGS84')
    # ...Albers Equal Area (Conic)
    elif projStyle=='AEA Conic':
        # Use the bounds of the stations for guidelines
        staLons,staLats=staLoc[:,1].astype(float),staLoc[:,2].astype(float)
        minLat,maxLat=np.min(staLats),np.max(staLats)
        minLon,maxLon=np.min(staLons),np.max(staLons)
        # Latitudes where the cone intersects the ellipsoid
        lat1,lat2=(maxLat-minLat)*1.0/6+minLat,(maxLat-minLat)*5.0/6+minLat
        # The origin of the projection
        lat0,lon0=(maxLat-minLat)*1.0/2+minLat,(maxLon-minLon)*1.0/2+minLon
        wantProj = pyproj.Proj(proj='aea',lat_1=lat1,lat_2=lat2,lat_0=lat0,lon_0=lon0,datum='WGS84',ellps='WGS84')
    else:
        print('projStyle '+projStyle+' not yet supported, skipped')
        return staLoc
    # Calculate the new X and Y values
    staLoc[:,1:3]=np.array([wantProj(float(lon),float(lat),inverse=False) for lon,lat in staLoc[:,1:3]],dtype=str)
    # Convert units from m to km
    staLoc[:,1:4]=staLoc[:,1:4].astype(float)/1000.0
    # Make the Z values positive downwards
    staLoc[:,3]=staLoc[:,3].astype(float)*-1.0
    return staLoc
    
    


# Author: Andrew.M.G.Reynen
from __future__ import print_function
from future.utils import iteritems

from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtCore import Qt

from Configuration import Ui_ConfDialog
from Actions import Action, ActionSetupDialog

# Configuration dialog
class ConfDialog(QtWidgets.QDialog, Ui_ConfDialog):
    def __init__(self,parent=None,main=None,actions=None,
                 pref=None,hotVar=None):
        QtWidgets.QDialog.__init__(self,parent)
        self.setupUi(self)
        self.main=main
        self.pref=pref
        self.act=actions
        self.hotVar=hotVar
        # Give the dialog some functionaly
        self.setFunctionality()
        # Load in the previous lists of preferences and actions
        self.loadLists()
        
    # Set up some functionality to the configuration dialog
    def setFunctionality(self):
        # Key press events (also includes left double click)
        self.confPrefList.keyPressedSignal.connect(self.prefListKeyEvent)
        self.confActiveList.keyPressedSignal.connect(self.actionListKeyEvent)
        self.confPassiveList.keyPressedSignal.connect(self.actionListKeyEvent)
        # Right click menus for the action lists
        self.confActiveList.setContextMenuPolicy(Qt.CustomContextMenu)
        self.confPassiveList.setContextMenuPolicy(Qt.CustomContextMenu)
        self.confActiveList.customContextMenuRequested.connect(self.createActionMenu)
        self.confPassiveList.customContextMenuRequested.connect(self.createActionMenu)
        # If the ordering of the passive list ever changes, update actPassiveOrder
        self.confPassiveList.leaveSignal.connect(self.updatePassiveOrder)
        # Add and delete action buttons
        self.confActiveAddButton.clicked.connect(lambda: self.addDelClicked('active','add'))
        self.confActiveDelButton.clicked.connect(lambda: self.addDelClicked('active','del'))
        self.confPassiveAddButton.clicked.connect(lambda: self.addDelClicked('passive','add'))
        self.confPassiveDelButton.clicked.connect(lambda: self.addDelClicked('passive','del'))
    
    # Load in all of the lists from previous state
    def loadLists(self):
        # Add the passive actions in order
        for key in self.main.actPassiveOrder:
            self.confPassiveList.addItem(self.actionItem(key))
        # Add the active actions (will be alphabetically ordered)
        for key in [key for key in self.act.keys() if not self.act[key].passive]:
            self.confActiveList.addItem(self.actionItem(key))
        # Add the preferences
        for key in self.pref.keys():
            item=QtWidgets.QListWidgetItem()
            item.setText(key)
            self.setItemSleepColor(item,False)
            item.setToolTip(self.pref[key].tip)
            self.confPrefList.addItem(item)
            
    # Handle add and delete action button click...
    # ...mocks key press events
    def addDelClicked(self,whichList,addDel):
        # Set the appropriate current list
        self.curList=self.confActiveList if whichList=='active' else self.confPassiveList
        # Set the current lists button press
        self.curList.key=Qt.Key_Insert if addDel=='add' else Qt.Key_Delete
        # Forward this to the key press event handles
        self.actionListKeyEvent(mock=True)
            
    # Return which action list is in focus
    def getCurActionList(self):
        curList=None
        if self.confActiveList.hasFocus() or self.confPassiveList.hasFocus():
            curList=self.confActiveList if self.confActiveList.hasFocus() else self.confPassiveList
        return curList

    # Function to handle calls to the preference lists
    def prefListKeyEvent(self):
        # If the preference list had focus (again "backspace" is triggered by double click)
        if self.confPrefList.key==Qt.Key_Backspace:
            # Grab the key, and update if possible
            item=self.confPrefList.currentItem()
            if item.isSelected():
                self.pref[item.text()].update(self)
        
    # Function to handle calls to the active and passive lists
    def actionListKeyEvent(self,mock=False):
        # If this was not a mock event (ie. actually triggered by key press)
        if not mock:
            self.curList=self.getCurActionList()
        # See if either of the action lists had focus, otherwise skip
        if self.curList==None:
            return
        # Skip if no accepted keys were passed
        if self.curList.key not in [Qt.Key_Insert,Qt.Key_Backspace,Qt.Key_Delete]:
            return
        # Creating a new action (Insert Key)
        if self.curList.key==Qt.Key_Insert:
            self.createAction()
            return
        # Skip if no action was selected
        if self.curList.currentItem() is None:
            return
        if not self.curList.currentItem().isSelected():
            return
        # Mark if the current action is currently in use
        inUse=(self.curList.currentItem().text() in list(self.main.qTimers.keys())+list(self.main.qThreads.keys()))   
        # Updating an action (Backspace Key -> which is triggered by double click)
        if self.curList.key==Qt.Key_Backspace:
            self.updateAction(inUse=inUse)
        # Delete an action (Delete Key)
        elif self.curList.key==Qt.Key_Delete:
            self.deleteAction(inUse=inUse)
        # Update the passive order every time (ie. do not care how configuration dialog is closed)...
        # ...a passive action is added or edited
        self.updatePassiveOrder()
        
    # Assign the tool tip to an action
    def actionItem(self,key):
        item=QtWidgets.QListWidgetItem()
        item.setText(key)
        # Set the tip (which is the trigger value)
        if not self.act[key].passive:
            try:
                item.setToolTip('Keybind: '+self.act[key].trigger.toString())
            except:
                item.setToolTip('Keybind: '+self.act[key].trigger)
        else:
            self.setPassiveTip(item)
        # Set the color of the item displaying the sleep state
        self.setItemSleepColor(item,self.act[key].sleeping)
        return item
    
    # Set a passive items tip
    def setPassiveTip(self,item):
        triggers=self.act[item.text()].trigger
        # Don't fill the entire screen if many triggers
        if len(triggers)>3:
            item.setToolTip('Activated by: '+','.join(triggers[:3]+['...']))
        else:
            item.setToolTip('Activated by: '+','.join(triggers))
            
    # Update the selected action from the specified list
    def updateAction(self,inUse=False):
        # Let user know if edits will be accepted later
        if inUse:
            print('Only trigger edits are allowed as action is strolling or scheming')
        # Open the action set-up dialog with selected action
        action=self.openActionSetup(self.act[self.curList.currentItem().text()],tempLock=inUse)
        # Do not update if no action returned or action is in use
        if action==None or inUse:
            return
        # Remove the old action
        self.act.pop(self.curList.currentItem().text())
        # Update the action dictionary with new
        self.act[action.tag]=action
        # If the action change had anything to do with an active action
        if (self.curList==self.confActiveList and action.passive) or (not action.passive):
            self.curList.takeItem(self.curList.currentRow())
            oList=self.confPassiveList if action.passive else self.confActiveList
            oList.addItem(self.actionItem(action.tag))
        # Otherwise just update the existing passive item (to preserve the passive order)
        else:
            self.curList.currentItem().setText(action.tag)
            self.setPassiveTip(self.curList.currentItem())
    
    # Open the action set-up dialog with a blank new action
    def createAction(self):
        # Make the new action
        if self.confActiveList.hasFocus():
            action=self.openActionSetup(Action(passive=False,trigger='Set Trigger'))
        else:
            action=self.openActionSetup(Action(passive=True,trigger=[]))
        self.insertLazyAction(action)
        
    # Insert a new action
    def insertLazyAction(self,action):
        if action==None:
            return
        # Insert new action to the action dictionary
        self.act[action.tag]=action
        # Create the item for the list widget
        item=self.actionItem(action.tag)
        # Add the action to the appropriate list
        if action.passive:
            self.confPassiveList.addItem(item)
        else:
            self.confActiveList.addItem(item)
    
    # Remove the selected action from the specified list
    def deleteAction(self,inUse=False):
        if inUse:
            print('Cannot delete an action which is strolling or scheming')
            return
        actTag=self.curList.currentItem().text()
        # If this is a locked action, the user cannot delete it
        if self.act[actTag].locked:
            print(actTag+' is a built-in action, it cannot be deleted')
            return
        # Remove from the list
        self.curList.takeItem(self.curList.currentRow())
        # As well as from the action dictionary
        self.act.pop(actTag)
    
    # Menu activated upon right clicking of action entries
    def createActionMenu(self,pos):
        # Get the key of the clicked on action
        self.curList=self.getCurActionList()
        item=self.curList.currentItem()
        if not item.isSelected():
            return
        self.menuAction=self.act[str(item.text())]
        # Can not edit locked actions, skip if locked        
        if self.menuAction.locked:
            print('Cannot sleep or copy locked actions')
            return
        # Create the menu, and fill with options...
        self.actionMenu= QtWidgets.QMenu()
        # ...sleep or awake
        if self.menuAction.sleeping:
            menuItem=self.actionMenu.addAction("Awake")
        else:
            menuItem=self.actionMenu.addAction("Sleep")
        menuItem.triggered.connect(self.toggleSleep)
        # ...copy the action
        menuItem=self.actionMenu.addAction("Copy")
        menuItem.triggered.connect(self.copyAction)
        # Move to cursor position and show
        parentPosition = self.curList.mapToGlobal(QtCore.QPoint(0, 0))       
        self.actionMenu.move(parentPosition + pos)
        self.actionMenu.show() 
    
    # Toggle the action sleeping state on/off
    def toggleSleep(self):
        item=self.curList.currentItem()
        if self.menuAction.sleeping:
            self.menuAction.sleeping=False
        else:
            self.menuAction.sleeping=True
        self.setItemSleepColor(item,self.menuAction.sleeping)
        
    # Set the color of a list widget item based on the actions sleeping state
    def setItemSleepColor(self,item,sleeping):
        if sleeping:
            item.setForeground(QtGui.QColor(150,150,150))
        else:
            item.setForeground(QtGui.QColor(40,40,40))
            
    # Copy an action, triggered from the action menu
    def copyAction(self):
        # Make a copy of the action
        newAction=Action()
        for key,val in iteritems(self.menuAction.__dict__):
            if key!='func':
                setattr(newAction,key,val)
        newAction.linkToFunction(self.main)
        # Set up the new actions tag
        seenKeys=self.act.keys()
        i=0
        while self.menuAction.tag+'('+str(i)+')' in seenKeys:
            i+=1
        # Assign the unique tag, and give a meaningless trigger (if an active action)
        newAction.tag=self.menuAction.tag+'('+str(i)+')'
        if not newAction.passive:
            print('Update '+newAction.tag+' trigger value, will be deleted upon configuration closure otherwise')
            newAction.trigger='Set Trigger'
        # Insert the copied action
        self.insertLazyAction(newAction)
    
    # Update the passive list ordering to what it seen by the user
    def updatePassiveOrder(self):
        self.main.actPassiveOrder=self.confPassiveList.visualListOrder()
                
    # Open the setup action dialog
    def openActionSetup(self,action,tempLock=False):
        self.dialog=ActionSetupDialog(self.main,action,self.act,
                                      self.hotVar,self.pref,tempLock)
        if self.dialog.exec_():
            action=self.dialog.returnAction()
            return action
    
    # On close, make sure that no new actions have incorrect triggers
    def closeEvent(self,ev):
        popKeys=[actKey for actKey in self.act.keys() if (self.act[actKey].trigger=='Set Trigger' and 
                                                          not self.act[actKey].passive)]
        for key in popKeys:
            print('Action '+key+' was removed, as its trigger value was not set')
            self.act.pop(key)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CustomPen.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_customPenDialog(object):
    def setupUi(self, customPenDialog):
        customPenDialog.setObjectName("customPenDialog")
        customPenDialog.resize(477, 338)
        self.verticalLayout = QtWidgets.QVBoxLayout(customPenDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tpTable = QtWidgets.QTableWidget(customPenDialog)
        self.tpTable.setObjectName("tpTable")
        self.tpTable.setColumnCount(0)
        self.tpTable.setRowCount(0)
        self.verticalLayout.addWidget(self.tpTable)
        self.tpButtonLayout = QtWidgets.QHBoxLayout()
        self.tpButtonLayout.setObjectName("tpButtonLayout")
        self.tpInsertButton = QtWidgets.QPushButton(customPenDialog)
        self.tpInsertButton.setObjectName("tpInsertButton")
        self.tpButtonLayout.addWidget(self.tpInsertButton)
        self.tpDeleteButton = QtWidgets.QPushButton(customPenDialog)
        self.tpDeleteButton.setObjectName("tpDeleteButton")
        self.tpButtonLayout.addWidget(self.tpDeleteButton)
        self.verticalLayout.addLayout(self.tpButtonLayout)

        self.retranslateUi(customPenDialog)
        QtCore.QMetaObject.connectSlotsByName(customPenDialog)

    def retranslateUi(self, customPenDialog):
        _translate = QtCore.QCoreApplication.translate
        customPenDialog.setWindowTitle(_translate("customPenDialog", "Custom Pen"))
        self.tpInsertButton.setText(_translate("customPenDialog", "Insert"))
        self.tpDeleteButton.setText(_translate("customPenDialog", "Delete"))




import numpy as np
from obspy import UTCDateTime
from scipy import signal

# Calculate the moving sum of an array
def MovingSum(arr, n=3):
    ret = np.cumsum(arr, dtype=float)
    ret[n:] = ret[n:] - ret[:-n]
    return ret[n - 1:]

# Calculate the cross correlation...
# ...which is (a*b)/(|a||b|) if normalized, just (a*b) otherwise
def calcXCor(template,data,norm=True):
    # Return nothing if the template is longer than the data
    if len(data)<=len(template)+1:
        return []
    Top=signal.fftconvolve(data, template[::-1], mode='valid')
    if norm:
        # Calculate the magnitudes of the data vectors to be used for normalization
        DataMag=np.sqrt(MovingSum(data**2,len(template)))
        Bot=DataMag*np.sqrt(np.sum(template**2))
        # Replace and bottom values of zero with the max of Bot ( the top values will also be affected, so...
        # ... areas near the zeros still won't get very large... just avoiding divide by zero here)
        Bot[Bot==0]=np.max(Bot) # These are always positive values, no need for np.abs()
        XCors=Top/Bot
    else:
        XCors=Top
    return XCors
    
# Gather the offset values, based on a specific template...
# ...assumes template is for a vertical channel
def getVertXCorOffsets(tempTrace,stream):
    stream=stream.select(component='Z')
    stas,offsets,xCors=[],[],[]
    # Taper the stream and template
    stream.taper(0.10,type='hann',max_length=2.0)
    tempTrace.taper(0.10,type='hann',max_length=2.0)
    # Loop through each trace and get their offsets, and max XCor values
    for tr in stream:
        # Skip if the trace in question does not have the same sampling rate
        if tr.stats.delta!=tempTrace.stats.delta:
            continue
        xCorArr=calcXCor(tempTrace.data,tr.data,norm=True)
        # Skip if the normalized cross-correlation values do not exist
        if len(xCorArr)==0:
            continue
        if np.max(xCorArr)<=0:
            continue
        aSta=tr.stats.network+'.'+tr.stats.station+'.'+tr.stats.location
        offset=tr.stats.starttime+tr.stats.delta*np.argmax(xCorArr)-tempTrace.stats.starttime
        xCor=np.max(xCorArr)
        # If another trace had the same station see if larger...
        if aSta in stas:
            idx=stas.index(aSta)
            # ...if so, replace previous values
            if xCor>xCors[idx]:
                xCors[idx]=xCor
                offsets[idx]=offset
        else:
            stas.append(aSta)
            offsets.append(offset)
            xCors.append(xCor)
    return stas,offsets
    
# Returns a stream, where the waveforms have been aligned to a specified reference...
# ...uses a template window length (in seconds) defined by [templatePre,templatePost]
# ...all alignment is based off the vertical channel
# ...the earliest pick of type "phaseType" is used as reference
def xCorAlign(*args,**kwargs):
    # Check to ensure all the wanted kwargs are present
    for kwarg in ['templatePre','templatePost','phaseType']:
        if kwarg not in kwargs.keys():
            print('Ensure that the optionals "templatePre","templatePost", and "phaseType" are defined')
            return '$pass'
    # Read the forced inputs
    pltSt=args[0].detrend('linear') # Have to atleast detrend for cross-correlation to work
    pickSet=args[1]
    phaseType=kwargs['phaseType']
    # If the stream or picks are empty, nothing to do
    if len(pltSt)==0:
        return '$pass'
    if len(pickSet)==0:
        print('No picks have been placed')
        return '$pass'
    # Gather the template to be used, taken from the first station with wanted phase type
    if phaseType not in pickSet[:,1]:
        print('No picks of phase type '+phaseType)
        return '$pass'
    else:
        wantArgs=np.where(pickSet[:,1]==phaseType)[0]
    # Sort by increasing pick time, and select the first station which is present in the stream... 
    # ...and has a vertical component
    stas=np.array([tr.stats.network+'.'+tr.stats.station+'.'+tr.stats.location for tr in pltSt])
    chas=np.array([tr.stats.channel[-1] for tr in pltSt])
    # Check to see that atleast two vertical channels are present
    if len(np.where(chas=='Z')[0])<2:
        print('Atleast two vertical traces are required for alignment')
        return '$pass'
    tempSta=''
    sortArgs=wantArgs[np.argsort(pickSet[wantArgs,2].astype(float))]
    for arg in sortArgs:
        # If station is present
        if pickSet[arg,0] in stas:
            # If channel is present
            if 'Z' in chas[np.where(stas==pickSet[arg,0])]:
                tempSta=pickSet[arg,0]
                pickTime=float(pickSet[arg,2])
                break
    # If no picks with the given phase had data, pass, and let user know
    if tempSta=='':
        print('No stations are present with phase type '+phaseType)
        return '$pass'
    # Gather the template to be used
    foundTemplate=False
    for arg in np.where((stas==tempSta)&(chas=='Z'))[0]:
        tempTr=pltSt[arg].slice(UTCDateTime(pickTime+kwargs['templatePre']),UTCDateTime(pickTime+kwargs['templatePost']))
        if tempTr.stats.npts*tempTr.stats.delta<0.5*(kwargs['templatePost']-kwargs['templatePre']):
            continue
        foundTemplate=True
        break
    # If no trace had alteast half of the wanted template, skip
    if not foundTemplate:
        print('The first pick of type '+phaseType+' which is on station '+tempSta+' did not have half'+
              ' of the desired template length')
        return '$pass'
    # Loop through all stations and get their offsets
    stas,offsets=getVertXCorOffsets(tempTr,pltSt.copy())
    missedStas=[]
    # Apply the offsets to the traces
    for tr in pltSt:
        aSta=tr.stats.network+'.'+tr.stats.station+'.'+tr.stats.location
        if aSta in stas:
            tr.stats.starttime-=offsets[stas.index(aSta)]
        else:
            missedStas.append(aSta)
    if len(missedStas)>0:
        print('The stations: '+str(np.unique(missedStas))+' were not time shifted')
    return pltSt


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ComboDialog.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_comboBoxDialog(object):
    def setupUi(self, comboBoxDialog):
        comboBoxDialog.setObjectName("comboBoxDialog")
        comboBoxDialog.resize(352, 84)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(comboBoxDialog.sizePolicy().hasHeightForWidth())
        comboBoxDialog.setSizePolicy(sizePolicy)
        comboBoxDialog.setMinimumSize(QtCore.QSize(0, 84))
        comboBoxDialog.setMaximumSize(QtCore.QSize(16777215, 84))
        self.verticalLayout = QtWidgets.QVBoxLayout(comboBoxDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.comboBox = QtWidgets.QComboBox(comboBoxDialog)
        self.comboBox.setObjectName("comboBox")
        self.verticalLayout.addWidget(self.comboBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(comboBoxDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        spacerItem = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(comboBoxDialog)
        self.buttonBox.accepted.connect(comboBoxDialog.accept)
        self.buttonBox.rejected.connect(comboBoxDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(comboBoxDialog)

    def retranslateUi(self, comboBoxDialog):
        _translate = QtCore.QCoreApplication.translate
        comboBoxDialog.setWindowTitle(_translate("comboBoxDialog", "Dialog"))




# Author: Andrew.M.G.Reynen
from __future__ import print_function
from copy import deepcopy
from future.utils import iteritems

from PyQt5 import QtWidgets,QtGui,QtCore
from PyQt5.QtCore import Qt

from CustomFunctions import dict2Text, text2Dict
from CustomPen import Ui_customPenDialog
from BasePen import Ui_basePenDialog
from ComboDialog import Ui_comboBoxDialog

# Default Preferences
def defaultPreferences(main):
    pref={
    'staPerPage':Pref(tag='staPerPage',val=6,dataType=int,
                      func=main.updateStaPerPage,condition={'bound':[1,30]},
                      tip='Number of stations (trace widgets) to display on each page'),
    'evePreTime':Pref(tag='evePreTime',val=-30,dataType=float,
                      tip='Time in seconds prior to the selected pick file names time to grab data'),
    'evePostTime':Pref(tag='evePostTime',val=60,dataType=float,
                      tip='Time in seconds after to the selected pick file names time to grab data'),
    'eveIdGenStyle':Pref(tag='eveIdGenStyle',val='next',dataType=str,
                         dialog='ComboBoxDialog',condition={'isOneOf':['fill','next']},
                         tip='Style used to generate a new empty pick files ID (when double clicking the archive event widget)'),
    'eveSortStyle':Pref(tag='eveSortStyle',val='time',dataType=str,
                        dialog='ComboBoxDialog',func=main.updateEveSort,condition={'isOneOf':['id','time']},
                        tip='How the archive list widget is sorted, also sorts hot variable pickFiles and pickTimes'),
    'cursorStyle':Pref(tag='cursorStyle',val='arrow',dataType=str,
                        dialog='ComboBoxDialog',func=main.updateCursor,condition={'isOneOf':['arrow','cross']},
                        tip='Cursor icon to use while hovering over plots'),
    'remExcessPicksStyle':Pref(tag='remExcessPicksStyle',val='oldest',dataType=str,
                        dialog='ComboBoxDialog',condition={'isOneOf':['oldest','closest','furthest']},
                        tip='Which excess pick(s) will be deleted when manually adding picks'),
    'staProjStyle':Pref(tag='staProjStyle',val='AEA Conic',dataType=str,
                        dialog='ComboBoxDialog',func=main.updateStaLoc,condition={'isOneOf':['None','AEA Conic','UTM']},
                        tip='Projection method to be applied when converting Lat,Lon to X,Y'),
    'pickTypesMaxCountPerSta':Pref(tag='pickTypesMaxCountPerSta',val={'P':1,'S':1},dataType=dict,
                                   func=main.updatePagePicks,condition={'bound':[1,999]},
                                   tip='Max number of picks of a given phase type allowed on any individual trace widget'),
    'basePen':Pref(tag='basePen',val={'widgetText':[14474460,1.0,0.0,False], # [Color,Width,Depth,UpdateMe]
                                      'traceBackground':[0,1.0,0.0,False],
                                      'timeBackground':[0,1.0,0.0,False],
                                      'imageBackground':[0,1.0,0.0,False],
                                      'imageBorder':[16777215,2.0,-1.0,False],
                                      'mapBackground':[0,1.0,0.0,False],
                                      'mapStaDefault':[16777215,4.0,0.0,False],
                                      'mapCurEve':[16776960,3.0,2.0,False],
                                      'mapPrevEve':[13107400,2.0,1.0,False],
                                      'mapPolygon':[3289800,1.0,10.0,False],
                                      'archiveBackground':[0,1.0,0.0,False],
                                      'archiveAvailability':[65280,1.0,0.0,False],
                                      'archiveSpanSelect':[3289800,1.0,0.0,False],
                                      'archiveCurEve':[16711680,3.0,1.0,False],
                                      'archivePrevEve':[48865,1.0,0.0,False],},
                    dataType=dict,dialog='BasePenDialog',func=main.updateBaseColors,
                    tip='Defines the base pen values for the main widgets'),
    'customPen':Pref(tag='customPen',val={'default':[16777215,1.0,0.0], # [Color,Width,Depth]
                                          'noStaData':[3289650,1.0,0.0],
                                          'noTraceData':[8224125,1.0,0.0],
                                          'goodMap':[65280,1.0,0.0],
                                          'poorMap':[16711680,1.0,0.0],
                                          'highlight':[255,1.0,2.0],
                                          'lowlight':[13158600,0.3,1.0],},dataType=dict,
                    dialog='CustomPenDialog',func=main.updateCustomPen,
                    tip='Defines the custom pen values referenced by PenAssign hot variables'),
    'pickPen':Pref(tag='pickPen',val={'default':[16777215,1.0,4.0],
                                      'P':[65280,1.0,5.0],
                                      'S':[16776960,1.0,5.0],},dataType=dict,
                    dialog='CustomPenDialog',func=main.updatePagePicks,
                    tip='Defines the pen values for the pick lines'),
    }
    return pref

# Capabilities for preferences
class Pref(object):
    def __init__(self,tag=None,val=None,dataType=str,
                 dialog='LineEditDialog',
                 func=None,condition={},tip=''):
        self.tag=tag # The preference key, and user visible name
        self.val=val # The preference value
        self.dataType=dataType # What kind of data is expected upon update
        self.dialog=dialog # The dialog which will pop up to return a value
        self.func=func # Function which is called on successful updates
        self.condition=condition # Key-word conditionals (see "LineEditDialog" in this file) 
        self.tip=tip # Short description of the preference
    
    # Return a deep copy of the objects value
    def getVal(self):
        val=deepcopy(self.val)
        return val
    
    # If the key was asked to be updated
    def update(self,hostWidget,init=False):
        # If this is the original initalization, don't ask for new value
        if not init:
            # Use the correct dialog...
            # ...text entry dialog
            if self.dialog=='LineEditDialog':
                if self.dataType==dict:
                    initText=dict2Text(self.val)
                else:
                    initText=str(self.val)
                val,ok=LineEditDialog.returnValue(tag=self.tag,initText=initText,
                                                  condition=self.condition,
                                                  dataType=self.dataType)
            # ...selecting from a list
            elif self.dialog=='ComboBoxDialog':
                val,ok=ComboBoxDialog.returnValue(self.val,self.condition['isOneOf'],self.tag)                    
            # ...default widget colors
            elif self.dialog=='BasePenDialog':
                BasePenDialog(self.val).exec_()
                # The checks and updates to the preference value are done within the dialog
                val,ok=self.val,True
            # ...custom colors and widths
            elif self.dialog=='CustomPenDialog':
                CustomPenDialog(self.val,self.tag).exec_()
                # The checks and updates to the preference value are done within the dialog
                val,ok=self.val,True
            else:
                print('New dialog?')
                val,ok=None,False
            # If the dialog was canceled, skip
            if not ok:
                return
            # If the no value was returned, skip
            if val==None:
                print('Value did not conform to '+str(self.dataType)+' and conditionals '+str(self.condition))
                return
            # Update the preference value
            self.val=val
        # If the value was updated, queue off its function
        if self.func!=None: 
            self.func(init=init)
            
# Dialog with line edit, allows for some initial text, and forced data type and condition
class LineEditDialog(QtWidgets.QDialog):
    def __init__(self,parent,tag,initText,condition,dataType):
        super(LineEditDialog, self).__init__(parent)
        self.setWindowTitle(tag)
        self.cond=condition
        self.dataType=dataType
        
        # Set up the layout and line edit
        layout = QtWidgets.QVBoxLayout(self)
        self.le = QtWidgets.QLineEdit(self)
        self.le.setText(initText)
        layout.addWidget(self.le)
        
        # OK and Cancel buttons
        self.buttons = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            Qt.Horizontal, self)
        layout.addWidget(self.buttons)
        self.buttons.accepted.connect(self.accept)
        self.buttons.rejected.connect(self.reject)
        
    # Get the text from the line edit
    def lineEditValue(self):
        val=self.le.text()
        # See first that the data type conforms
        try:
            # Allow user to type in the dictionary a bit easier
            if self.dataType==dict:
                val=text2Dict(val)
                if val=={}:
                    print('No keys are present in the dictionary')
                    return None
            else:
                val=self.dataType(val)
        except:
            return None
        # Loop through all values (usually just one, can be more for dictionary)
        if self.dataType==dict:
            vals=[aVal for key,aVal in iteritems(val)]
        else:
            vals=[val]
        # ...and check against the built in conditionals
        keys=[key for key in self.cond.keys()]
        for aVal in vals:
            if 'bound' in keys:
                if aVal<self.cond['bound'][0] or aVal>self.cond['bound'][1]:
                    return None
        return val

    # Static method to create the dialog and return value
    @staticmethod
    def returnValue(parent=None,tag='',initText='',condition={},dataType=str):
        dialog = LineEditDialog(parent,tag,initText,condition,dataType)
        result = dialog.exec_()
        return dialog.lineEditValue(), result==QtWidgets.QDialog.Accepted


# Dialog window for editing the basic widget colors
class BasePenDialog(QtWidgets.QDialog, Ui_basePenDialog):
    def __init__(self,tpDict,parent=None):
        QtWidgets.QDialog.__init__(self,parent)
        self.setupUi(self)
        self.tpDict=tpDict
        self.keyOrder=sorted(self.tpDict.keys()) # Used to reference back before the last user change
        # Fill in the current customPen information (before setting functionality, as functionality has a "onChanged" signal)
        self.fillDialog()
        # Give the dialog some functionaly
        self.setFunctionality()
        
    # Set up some functionality to the custom pen dialog
    def setFunctionality(self):
        self.tpTable.itemDoubleClicked.connect(self.updatePenColor)
        self.tpTable.itemChanged.connect(self.updateItemText)

    # Fill the dialog with info relating to the current customPen dictionary
    def fillDialog(self):
        # Make the table large enough to hold all current information
        self.tpTable.setRowCount(len(self.tpDict))
        self.tpTable.setColumnCount(4)
        # Set the headers
        self.tpTable.setHorizontalHeaderLabels(['Tag','Color','Width','Depth'])     
        # Enter data onto Table
        for m,key in enumerate(sorted(self.tpDict.keys())):
            # Generate the table items...
            tagItem=QtWidgets.QTableWidgetItem(key)
            tagItem.setFlags(Qt.ItemIsEnabled)
            penItem=QtWidgets.QTableWidgetItem('')
            penItem.setFlags(Qt.ItemIsEnabled)
            penItem.setBackground(QtGui.QColor(self.tpDict[key][0]))
            widItem=QtWidgets.QTableWidgetItem(str(self.tpDict[key][1]))
            depItem=QtWidgets.QTableWidgetItem(str(self.tpDict[key][2]))
            # Put items in wanted position
            self.tpTable.setItem(m,0,tagItem)
            self.tpTable.setItem(m,1,penItem)
            self.tpTable.setItem(m,2,widItem)
            self.tpTable.setItem(m,3,depItem)
        # Resize the dialog
        self.tpTable.resizeColumnsToContents()
            
    # Update the pen color for a given tag
    def updatePenColor(self,item):
        if item.column()!=1:
            return
        itemKey=self.keyOrder[item.row()]
        # Go get a new color
        colorDialog=QtWidgets.QColorDialog()
        val=colorDialog.getColor(QtGui.QColor(self.tpDict[itemKey][0]),self)
        # Set the new color (if the dialog was not canceled)
        if val.isValid():
            val=val.rgba()
            item.setBackground(QtGui.QColor(val))
            self.tpDict[itemKey][0]=val
            # Mark that this item was edited
            self.tpDict[itemKey][3]=True
            
    # Update the text values, if changed
    def updateItemText(self,item):
        if item.column()in [0,1]:
            return
        # The key, prior to changes
        itemKey=self.keyOrder[item.row()]
        # Updating the width or depth values
        if item.column() in [2,3]:
            prefIdx=item.column()-1
            try:
                val=float(item.text())
            except:
                val=-99999
            # Ensure the width value is reasonable, if not change back the original
            if prefIdx==1 and (val<0 or val>10):
                print('Width should be in the range [0,10]')
                item.setText(str(self.tpDict[itemKey][prefIdx]))
            elif prefIdx==2 and (val<-10 or val>=10):
                print('Depth should be in the range [-10,10)')
                item.setText(str(self.tpDict[itemKey][prefIdx]))
            # If passed checks, update the tpDict with the new width or depth
            else:
                self.tpDict[itemKey][prefIdx]=float(item.text())
                # Mark that this item was edited
                self.tpDict[itemKey][3]=True

# Dialog window for editing the custom colors and widths
class CustomPenDialog(QtWidgets.QDialog, Ui_customPenDialog):
    def __init__(self,tpDict,tag,parent=None):
        QtWidgets.QDialog.__init__(self,parent)
        self.setupUi(self)
        self.setWindowTitle(tag)
        self.tpDict=tpDict
        self.keyOrder=sorted(self.tpDict.keys()) # Used to reference back before the last user change
        # Fill in the current customPen information (before setting functionality, as functionality has a "onChanged" signal)
        self.fillDialog()
        # Give the dialog some functionaly
        self.setFunctionality()
        
    # Set up some functionality to the custom pen dialog
    def setFunctionality(self):
        self.tpTable.itemDoubleClicked.connect(self.updatePenColor)
        self.tpTable.itemChanged.connect(self.updateItemText)
        self.tpInsertButton.clicked.connect(self.insertPen)
        self.tpDeleteButton.clicked.connect(self.deletePen)

    # Fill the dialog with info relating to the current customPen dictionary
    def fillDialog(self):
        # Make the table large enough to hold all current information
        self.tpTable.setRowCount(len(self.tpDict))
        self.tpTable.setColumnCount(4)
        # Set the headers
        self.tpTable.setHorizontalHeaderLabels(['Tag','Color','Width','Depth'])     
        # Enter data onto Table
        for m,key in enumerate(sorted(self.tpDict.keys())):
            # Generate the table items...
            tagItem=QtWidgets.QTableWidgetItem(key)
            if key=='default':
                tagItem.setFlags(Qt.ItemIsEnabled)
            penItem=QtWidgets.QTableWidgetItem('')
            penItem.setFlags(Qt.ItemIsEnabled)
            penItem.setBackground(QtGui.QColor(self.tpDict[key][0]))
            widItem=QtWidgets.QTableWidgetItem(str(self.tpDict[key][1]))
            depItem=QtWidgets.QTableWidgetItem(str(self.tpDict[key][2]))
            # Put items in wanted position
            self.tpTable.setItem(m,0,tagItem)
            self.tpTable.setItem(m,1,penItem)
            self.tpTable.setItem(m,2,widItem)
            self.tpTable.setItem(m,3,depItem)
    
    # Update the pen color for a given tag
    def updatePenColor(self,item):
        if item.column()!=1:
            return
        itemKey=self.keyOrder[item.row()]
        # Go get a new color
        colorDialog=QtWidgets.QColorDialog()
        val=colorDialog.getColor(QtGui.QColor(self.tpDict[itemKey][0]),self)
        # Set the new color (if the dialog was not canceled)
        if val.isValid():
            val=val.rgba()
            item.setBackground(QtGui.QColor(val))
            self.tpDict[itemKey][0]=val
    
    # Update the text values, if changed
    def updateItemText(self,item):
        if item.column()==1:
            return
        # Disconnect itself, changes occur here weird looping otherwise
        self.tpTable.itemChanged.disconnect(self.updateItemText)
        # The key, prior to changes
        itemKey=self.keyOrder[item.row()]
        # Updating the width or depth values
        if item.column() in [2,3]:
            prefIdx=item.column()-1
            try:
                val=float(item.text())
            except:
                val=-99999
            # Ensure the width value is reasonable, if not change back the original
            if prefIdx==1 and (val<0 or val>10):
                print('Width should be in the range [0,10]')
                item.setText(str(self.tpDict[itemKey][prefIdx]))
            elif prefIdx==2 and (val<-10 or val>=10):
                print('Depth should be in the range [-10,10)')
                item.setText(str(self.tpDict[itemKey][prefIdx]))
            # If passed checks, update the tpDict with the new width or depth
            else:
                self.tpDict[itemKey][prefIdx]=float(item.text())
        # Updating the tag
        elif item.column()==0:
            # If the same, do nothing
            if item.text()==itemKey:
                pass
            # Do not allow duplicate tags
            elif item.text() in [key for key in self.tpDict.keys() if key!=itemKey]:
                print('This tag is already present, update denied')
                item.setText(itemKey)
            else:
                # If passed checks, update the tpDict with the new tag
                self.tpDict[str(item.text())]=self.tpDict[itemKey]
                self.tpDict.pop(itemKey)
                # Update the key order, so can reference back before changes
                self.keyOrder[item.row()]=str(item.text())
        # Reconnect to OnChanged signal
        self.tpTable.itemChanged.connect(self.updateItemText)
            
    # Insert a new pen
    def insertPen(self):        
        # If the "NewPen" key is still present, ask to change it...
        if 'NewPen' in [key for key in self.tpDict.keys()]:
            print('Change the tag "NewPen", to be able to add another custom pen"')
            return
        # Disconnect the OnChanged signal (as this would trigger it)
        self.tpTable.itemChanged.disconnect(self.updateItemText)
        # Add a row with the default parameters
        m=self.tpTable.rowCount()
        self.tpTable.setRowCount(m+1)
        self.tpTable.setItem(m,0,QtWidgets.QTableWidgetItem('NewPen'))
        penItem=QtWidgets.QTableWidgetItem('')
        penItem.setFlags(Qt.ItemIsEnabled)
        self.tpTable.setItem(m,1,penItem)
        self.tpTable.setItem(m,2,QtWidgets.QTableWidgetItem('1.0'))
        self.tpTable.setItem(m,3,QtWidgets.QTableWidgetItem('0.0'))
        # Add it also to the dictionary
        self.tpDict['NewPen']=[4294967295,1.0,0.0]
        # Update the keyOrder
        self.keyOrder.append('NewPen')
        # Reconnect to OnChanged signal
        self.tpTable.itemChanged.connect(self.updateItemText)
        
    # Delete the currently selected pen
    def deletePen(self):
        idx=self.tpTable.currentRow()
        key=str(self.tpTable.item(idx,0).text())
        # Not allowed to delete default
        if key=='default':
            print('Cannot delete the pen tagged default')
            return
        # Disconnect the OnChanged signal (as this would trigger it)
        self.tpTable.itemChanged.disconnect(self.updateItemText)
        self.tpTable.removeRow(idx)
        self.keyOrder.pop(idx)
        self.tpDict.pop(key)
        # Reconnect to OnChanged signal
        self.tpTable.itemChanged.connect(self.updateItemText)
        
# Dialog to get a specific date time back
class DateDialog(QtWidgets.QDialog):
    def __init__(self, parent = None):
        super(DateDialog, self).__init__(parent)
        
        # Give a window title
        self.setWindowTitle('Date Time Select')

        layout = QtWidgets.QVBoxLayout(self)
        # Widget for editing the date
        self.datetime = QtWidgets.QDateTimeEdit(self)
        self.datetime.setDisplayFormat('yyyy-MM-dd hh:mm:ss')
        layout.addWidget(self.datetime)

        # OK and Cancel buttons
        buttons = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            Qt.Horizontal, self)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    # Get current date and time from the dialog
    def dateTime(self):
        return self.datetime.dateTime()

    # Static method to create the dialog and return [timestamp, accepted]
    @staticmethod
    def getDateTime(bound,parent = None):
        # Start dialog
        dialog = DateDialog(parent)
        # Set the previous string value
        prevDateTime=QtCore.QDateTime()
        prevDateTime.setTimeSpec(Qt.UTC)
        prevDateTime.setTime_t(int(bound))
        dialog.datetime.setDateTime(prevDateTime)
        # Get and return value
        result = dialog.exec_()
        dateTime=dialog.dateTime()
        dateTime.setTimeSpec(Qt.UTC)
        newBound = dateTime.toTime_t()
        return newBound, result == QtWidgets.QDialog.Accepted
        
# Dialog window for selecting one of a few options
class ComboBoxDialog(QtWidgets.QDialog, Ui_comboBoxDialog):
    def __init__(self,parent,curVal,acceptVals,tag):
        QtWidgets.QDialog.__init__(self,parent)
        self.setupUi(self)
        self.setWindowTitle(tag)
        # Fill in the combo dialog box
        self.comboBox.addItems(acceptVals)
        # Set the current text to be the current preference value
        index = self.comboBox.findText(curVal)
        if index >= 0:
            self.comboBox.setCurrentIndex(index)
        
    # Static method to create the dialog and return the selected value
    @staticmethod
    def returnValue(curVal,acceptVals,tag,parent=None):
        dialog = ComboBoxDialog(parent,curVal,acceptVals,tag)
        result = dialog.exec_()
        return dialog.comboBox.currentText(), result==QtWidgets.QDialog.Accepted


#from setuptools import setup
from distutils.core import setup
import inspect
import os

from lazylyst.__init__ import __version__

SETUP_DIRECTORY = os.path.dirname(os.path.abspath(inspect.getfile(
    inspect.currentframe())))
walkPath=os.path.join(SETUP_DIRECTORY, 'lazylyst')

def getPackages():
    modules = []
    for dirPath, _, fileNames in os.walk(walkPath):
        if '__init__.py' in fileNames:
            modules.append(os.path.relpath(dirPath, SETUP_DIRECTORY))
    return [_i.replace(os.sep, ".") for _i in modules]

def getResources():
    resources=[]
    for dirPath, _, fileNames in os.walk(walkPath):
        for aFile in fileNames:
            if (aFile.split('.')[-1] in ['txt','mseed','png','xml','picks','npy'] or 
                aFile=='setGen.ini'):
                resources.append(os.path.join(os.path.relpath(dirPath, walkPath),aFile))
    return resources

setup(
  name = 'lazylyst',
  version = __version__,
  description = 'GUI for timeseries review with a focus on seismology',
  author = 'Andrew Reynen',
  author_email = 'andrew.m.g.reynen@gmail.com',
  license='MIT License',
  url = 'https://github.com/AndrewReynen/Lazylyst', 
  keywords = ['seismology', 'timeseries', 'picking', 'pyqtgraph'], 
  classifiers = [],
  packages=getPackages(),
  package_data={'lazylyst': getResources()},
  include_package_data=True,
  install_requires=[
        'obspy>=1.0.2',
        'pyqtgraph==0.10.0',
        'scandir>=1.4',
        'pyproj>=1.9.5.1',
        'future',
        'sip'
    ],
)



from __future__ import print_function
import warnings

import numpy as np
import scipy.optimize as optimize
warnings.simplefilter("ignore", optimize.OptimizeWarning)

# Get the velocity and delay values based on the current source...
# ...used for the simple locator (see simpleLocatorFunc for the equation, units in km and s)
# ...Vp=P-velocity,Vs=S-velocity,Dp=P-delay,Ds=S-delay
# ...tResThresh is the time residual which is residual value beyond which is considered poor
def getVelDelay(sourceDict):
    # Ensure all required values are present
    if 'Velocity' not in sourceDict.keys():
        return '$pass'
    velDict={}
    for key in ['Vp','Vs','Dp','Ds','tResThresh']:
        if key not in sourceDict['Velocity'].keys():
            return '$pass'
        velDict[key]=sourceDict['Velocity'][key]
    return velDict

# Function to calculate the arrival times
# ti=function((xi,yi,zi,vi,di),x0,y0,t0)
# ti=t0+(Dist+Vel*Delay)/Vel
def simpleLocatorFunc(data,x0,y0,z0,t0):
    return t0+((((data[:,0]-x0)**2+(data[:,1]-y0)**2+(data[:,2]-z0)**2)**0.5)+data[:,3]*data[:,4])/data[:,3]

# Gather the information to be sent to the location minimization function
def getPickData(pickSet,staLoc,vdInfo):
    # Form array to be able to send to the 0 dimensional travel time function
    data,stas=[],[] #[xi,yi,zi,vi,ti],[]
    for aPick in pickSet:
        # Ensure that this pick has station metadata present
        idx=np.where(staLoc[:,0]==aPick[0])[0]
        if len(idx)<1:
            continue
        aStaX,aStaY,aStaZ=staLoc[idx[0],1:4].astype(float)
        # Assign appropriate velocity values        
        if aPick[1]=='P':
            aVel=vdInfo['Vp']
            aDelay=vdInfo['Dp']
        elif aPick[1]=='S':
            aVel=vdInfo['Vs']
            aDelay=vdInfo['Ds']
        else:
            continue
        aTime=float(aPick[2]) ## Give more sig digs?... rough approximation anyways
        data.append([aStaX,aStaY,aStaZ,aVel,aDelay,aTime])
        stas.append(aPick[0])
    return np.array(data),np.array(stas,dtype=str)

# Locate event using a straight ray path, and non-linear least squares curve fitting
# Also assign residual coloring to the stations
# Will not return a location if no velocities supplied, or stations are not projected
def simpleLocator(pickSet,staLoc,mapCurEve,staSort,sourceDict,staProjStyle):
    # Use only P and S picks
    if len(pickSet)>=4:
        pickSet=pickSet[np.where((pickSet[:,1]=='P')|(pickSet[:,1]=='S'))]
    # Set up the pen assignment arrays
    if len(staLoc)==0:
        traceBgPenAssign={'noStaData':list(staSort)}
    else:
        traceBgPenAssign={'noStaData':[sta for sta in staSort if sta not in staLoc[:,0]]}
    if len(staSort)==0:
        mapStaPenAssign={'noTraceData':[row[0] for row in staLoc]}
    else:
        mapStaPenAssign={'noTraceData':[row[0] for row in staLoc if row[0] not in staSort]}
    mapStaPenAssign['goodMap']=list(np.unique(pickSet[:,0]))
    # If there is no vpInfo defined or the stations are not projected, do not update the location
    vdInfo=getVelDelay(sourceDict)
    if vdInfo=='$pass' or staProjStyle=='None':
        return '$pass',traceBgPenAssign,mapStaPenAssign
    data,stas=getPickData(pickSet,staLoc,vdInfo)    
    # If there are too few picks, do not try to locate
    if len(data)<4:
        return np.empty((0,5)),traceBgPenAssign,mapStaPenAssign
    # Compute value to take away from the very large travel times (easier on the curve fitting)
    Tref=np.min(pickSet[:,2].astype(float))
    data[:,5]-=Tref
    # Solve for event origin parameters
    try:
        params, pcov = optimize.curve_fit(simpleLocatorFunc, data[:,:5], data[:,5],(0,0,0,0))
        x0,y0,z0,t0=params
    except:
        print('simpleLocator failed')
        return np.empty((0,5)),traceBgPenAssign,mapStaPenAssign
    # Get the residual values...
    # ...predicted arrival times
    ti=simpleLocatorFunc(data[:,:5],x0,y0,z0,t0)
    # Get the difference with the actual arrival times
    resids=np.abs(ti-data[:,5])
    poorResidStas=np.unique(stas[np.where(resids>=vdInfo['tResThresh'])])
    goodResidStas=np.unique(stas[np.where(resids<vdInfo['tResThresh'])])
    # Add the good/bad stations to proper pen assignments
    mapStaPenAssign['poorMap']=list(poorResidStas)
    mapStaPenAssign['goodMap']=list(goodResidStas)
    # Add back the time offset
    t0=Tref+t0
    return np.array([[0,x0,y0,z0,t0]]),traceBgPenAssign,mapStaPenAssign


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(871, 785)
        MainWindow.setDockNestingEnabled(True)
        self.mainLayout = QtWidgets.QWidget(MainWindow)
        self.mainLayout.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mainLayout.sizePolicy().hasHeightForWidth())
        self.mainLayout.setSizePolicy(sizePolicy)
        self.mainLayout.setMinimumSize(QtCore.QSize(0, 0))
        self.mainLayout.setObjectName("mainLayout")
        MainWindow.setCentralWidget(self.mainLayout)
        self.textOutDock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textOutDock.sizePolicy().hasHeightForWidth())
        self.textOutDock.setSizePolicy(sizePolicy)
        self.textOutDock.setMinimumSize(QtCore.QSize(181, 155))
        self.textOutDock.setMaximumSize(QtCore.QSize(524287, 524287))
        self.textOutDock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.textOutDock.setObjectName("textOutDock")
        self.textOutLayout = QtWidgets.QWidget()
        self.textOutLayout.setObjectName("textOutLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.textOutLayout)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.textOutBrowser = QtWidgets.QTextBrowser(self.textOutLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textOutBrowser.sizePolicy().hasHeightForWidth())
        self.textOutBrowser.setSizePolicy(sizePolicy)
        self.textOutBrowser.setObjectName("textOutBrowser")
        self.verticalLayout_2.addWidget(self.textOutBrowser)
        self.textOutGridLayout = QtWidgets.QGridLayout()
        self.textOutGridLayout.setObjectName("textOutGridLayout")
        self.strollingLabel = QtWidgets.QLabel(self.textOutLayout)
        self.strollingLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.strollingLabel.setObjectName("strollingLabel")
        self.textOutGridLayout.addWidget(self.strollingLabel, 0, 0, 1, 1)
        self.strollComboBox = QtWidgets.QComboBox(self.textOutLayout)
        self.strollComboBox.setInsertPolicy(QtWidgets.QComboBox.InsertAlphabetically)
        self.strollComboBox.setObjectName("strollComboBox")
        self.textOutGridLayout.addWidget(self.strollComboBox, 1, 0, 1, 1)
        self.schemingLabel = QtWidgets.QLabel(self.textOutLayout)
        self.schemingLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.schemingLabel.setObjectName("schemingLabel")
        self.textOutGridLayout.addWidget(self.schemingLabel, 0, 1, 1, 1)
        self.schemeComboBox = QtWidgets.QComboBox(self.textOutLayout)
        self.schemeComboBox.setInsertPolicy(QtWidgets.QComboBox.InsertAlphabetically)
        self.schemeComboBox.setObjectName("schemeComboBox")
        self.textOutGridLayout.addWidget(self.schemeComboBox, 1, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.textOutGridLayout)
        self.textOutDock.setWidget(self.textOutLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.textOutDock)
        self.archiveListDock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveListDock.sizePolicy().hasHeightForWidth())
        self.archiveListDock.setSizePolicy(sizePolicy)
        self.archiveListDock.setMinimumSize(QtCore.QSize(96, 131))
        self.archiveListDock.setMaximumSize(QtCore.QSize(524287, 524287))
        self.archiveListDock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.archiveListDock.setObjectName("archiveListDock")
        self.archiveListLayout = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveListLayout.sizePolicy().hasHeightForWidth())
        self.archiveListLayout.setSizePolicy(sizePolicy)
        self.archiveListLayout.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.archiveListLayout.setObjectName("archiveListLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.archiveListLayout)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.archiveList = ArchiveListWidget(self.archiveListLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveList.sizePolicy().hasHeightForWidth())
        self.archiveList.setSizePolicy(sizePolicy)
        self.archiveList.setMinimumSize(QtCore.QSize(0, 0))
        self.archiveList.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.archiveList.setObjectName("archiveList")
        self.verticalLayout.addWidget(self.archiveList)
        self.archiveListLineEdit = QtWidgets.QLineEdit(self.archiveListLayout)
        self.archiveListLineEdit.setCursor(QtGui.QCursor(QtCore.Qt.IBeamCursor))
        self.archiveListLineEdit.setObjectName("archiveListLineEdit")
        self.verticalLayout.addWidget(self.archiveListLineEdit)
        self.archiveListDock.setWidget(self.archiveListLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.archiveListDock)
        self.mapDock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mapDock.sizePolicy().hasHeightForWidth())
        self.mapDock.setSizePolicy(sizePolicy)
        self.mapDock.setMinimumSize(QtCore.QSize(78, 103))
        self.mapDock.setMaximumSize(QtCore.QSize(524287, 524287))
        self.mapDock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.mapDock.setObjectName("mapDock")
        self.mayLayout = QtWidgets.QWidget()
        self.mayLayout.setObjectName("mayLayout")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.mayLayout)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setSpacing(1)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.mapWidget = MapWidget(self.mayLayout)
        self.mapWidget.setObjectName("mapWidget")
        self.verticalLayout_6.addWidget(self.mapWidget)
        self.mapDock.setWidget(self.mayLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.mapDock)
        self.archiveViewDock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveViewDock.sizePolicy().hasHeightForWidth())
        self.archiveViewDock.setSizePolicy(sizePolicy)
        self.archiveViewDock.setMinimumSize(QtCore.QSize(78, 126))
        self.archiveViewDock.setMaximumSize(QtCore.QSize(524287, 524287))
        self.archiveViewDock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.archiveViewDock.setObjectName("archiveViewDock")
        self.archiveViewLayout = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveViewLayout.sizePolicy().hasHeightForWidth())
        self.archiveViewLayout.setSizePolicy(sizePolicy)
        self.archiveViewLayout.setObjectName("archiveViewLayout")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.archiveViewLayout)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.archiveViewTextLayout = QtWidgets.QHBoxLayout()
        self.archiveViewTextLayout.setSpacing(0)
        self.archiveViewTextLayout.setObjectName("archiveViewTextLayout")
        self.archiveSpanT0Label = DblClickLabelWidget(self.archiveViewLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveSpanT0Label.sizePolicy().hasHeightForWidth())
        self.archiveSpanT0Label.setSizePolicy(sizePolicy)
        self.archiveSpanT0Label.setMinimumSize(QtCore.QSize(0, 10))
        self.archiveSpanT0Label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.archiveSpanT0Label.setFont(font)
        self.archiveSpanT0Label.setText("")
        self.archiveSpanT0Label.setObjectName("archiveSpanT0Label")
        self.archiveViewTextLayout.addWidget(self.archiveSpanT0Label)
        self.archiveSpanT1Label = DblClickLabelWidget(self.archiveViewLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveSpanT1Label.sizePolicy().hasHeightForWidth())
        self.archiveSpanT1Label.setSizePolicy(sizePolicy)
        self.archiveSpanT1Label.setMinimumSize(QtCore.QSize(0, 10))
        self.archiveSpanT1Label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.archiveSpanT1Label.setFont(font)
        self.archiveSpanT1Label.setText("")
        self.archiveSpanT1Label.setObjectName("archiveSpanT1Label")
        self.archiveViewTextLayout.addWidget(self.archiveSpanT1Label)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.archiveViewTextLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.archiveViewTextLayout)
        self.archiveSpan = ArchiveSpanWidget(self.archiveViewLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveSpan.sizePolicy().hasHeightForWidth())
        self.archiveSpan.setSizePolicy(sizePolicy)
        self.archiveSpan.setMinimumSize(QtCore.QSize(0, 20))
        self.archiveSpan.setMaximumSize(QtCore.QSize(16777215, 65))
        self.archiveSpan.setObjectName("archiveSpan")
        self.verticalLayout_3.addWidget(self.archiveSpan)
        self.archiveEvent = ArchiveEventWidget(self.archiveViewLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.archiveEvent.sizePolicy().hasHeightForWidth())
        self.archiveEvent.setSizePolicy(sizePolicy)
        self.archiveEvent.setMinimumSize(QtCore.QSize(0, 20))
        self.archiveEvent.setMaximumSize(QtCore.QSize(16777215, 40))
        self.archiveEvent.setObjectName("archiveEvent")
        self.verticalLayout_3.addWidget(self.archiveEvent)
        spacerItem1 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.archiveViewDock.setWidget(self.archiveViewLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(4), self.archiveViewDock)
        self.traceViewDock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.traceViewDock.sizePolicy().hasHeightForWidth())
        self.traceViewDock.setSizePolicy(sizePolicy)
        self.traceViewDock.setMinimumSize(QtCore.QSize(80, 82))
        self.traceViewDock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.traceViewDock.setObjectName("traceViewDock")
        self.traceViewLayout = QtWidgets.QWidget()
        self.traceViewLayout.setObjectName("traceViewLayout")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.traceViewLayout)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.traceSplitter = QtWidgets.QSplitter(self.traceViewLayout)
        self.traceSplitter.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.traceSplitter.setLineWidth(0)
        self.traceSplitter.setOrientation(QtCore.Qt.Vertical)
        self.traceSplitter.setHandleWidth(1)
        self.traceSplitter.setChildrenCollapsible(False)
        self.traceSplitter.setObjectName("traceSplitter")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.traceSplitter)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.timeImageLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.timeImageLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.timeImageLayout.setContentsMargins(0, 0, 0, 0)
        self.timeImageLayout.setSpacing(1)
        self.timeImageLayout.setObjectName("timeImageLayout")
        self.timeWidget = TimeWidget(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeWidget.sizePolicy().hasHeightForWidth())
        self.timeWidget.setSizePolicy(sizePolicy)
        self.timeWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.timeWidget.setMaximumSize(QtCore.QSize(16777215, 50))
        self.timeWidget.setBaseSize(QtCore.QSize(0, 0))
        self.timeWidget.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.timeWidget.setObjectName("timeWidget")
        self.timeImageLayout.addWidget(self.timeWidget)
        self.imageWidget = ImageWidget(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imageWidget.sizePolicy().hasHeightForWidth())
        self.imageWidget.setSizePolicy(sizePolicy)
        self.imageWidget.setMinimumSize(QtCore.QSize(0, 1))
        self.imageWidget.setObjectName("imageWidget")
        self.timeImageLayout.addWidget(self.imageWidget)
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.traceSplitter)
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.traceLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.traceLayout.setContentsMargins(0, 0, 0, 0)
        self.traceLayout.setSpacing(1)
        self.traceLayout.setObjectName("traceLayout")
        self.verticalLayout_4.addWidget(self.traceSplitter)
        self.traceViewDock.setWidget(self.traceViewLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.traceViewDock)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Lazylyst"))
        self.textOutDock.setWindowTitle(_translate("MainWindow", "Standard Out"))
        self.strollingLabel.setText(_translate("MainWindow", "Strolling (0)"))
        self.schemingLabel.setText(_translate("MainWindow", "Scheming (0)"))
        self.archiveListDock.setWindowTitle(_translate("MainWindow", "Archive List"))
        self.archiveListLineEdit.setPlaceholderText(_translate("MainWindow", "Search"))
        self.mapDock.setWindowTitle(_translate("MainWindow", "Map View"))
        self.archiveViewDock.setWindowTitle(_translate("MainWindow", "Archive View"))
        self.traceViewDock.setWindowTitle(_translate("MainWindow", "Trace View"))

from CustomWidgets import DblClickLabelWidget
from MapWidgets import MapWidget
from TemporalWidgets import ArchiveEventWidget, ArchiveListWidget, ArchiveSpanWidget, ImageWidget, TimeWidget
